function safeSendMessage(message) {
  try {
    chrome.runtime.sendMessage(message, (response) => {
      if (chrome.runtime.lastError) {
        console.error('SendMessage Error:', chrome.runtime.lastError.message);
      } else {
        //console.log('Message sent successfully.');
      }
    });
  } catch (e) {
    console.warn('Extension context invalidated. Cannot send message.', e);
  }
}
/**
async function sendCanvasToBackground() {
  const blob = await captureChartCanvas();
  if (!blob) return;

  // Let user download it locally
  downloadBlob(blob, `tv-screenshot-${Date.now()}.png`);

  // Then upload to Supabase
  const reader = new FileReader();
  reader.onloadend = () => {
    const base64Data = reader.result.split(',')[1];
    chrome.runtime.sendMessage({
      type: 'UPLOAD_SCREENSHOT',
      imageBase64: base64Data,
      filename: `tv-screenshot-${Date.now()}.png`
    });
  };
  reader.readAsDataURL(blob);
}
 */
function downloadBlob(blob, filename = 'tv-screenshot.png') {
  const url = URL.createObjectURL(blob);
  const a = document.createElement('a');
  a.href = url;
  a.download = filename;
  document.body.appendChild(a);
  a.click();
  document.body.removeChild(a);
  URL.revokeObjectURL(url);
}

async function captureChartCanvas() {
  const canvas = document.querySelector('canvas');

  if (!canvas) {
    console.warn('No canvas element found.');
    return null;
  }
  return new Promise((resolve) => {
    canvas.toBlob((blob) => {
      if (!blob) {
        console.error('Canvas capture failed.');
        resolve(null);
      } else {
        resolve(blob);
      }
    }, 'image/png');
  });
}

function createCaptureButton() {
  if (document.getElementById('capture-trade-button')) return;

  const button = document.createElement('button');
  button.id = 'capture-trade-button';
  button.innerText = 'Capture Trade';

  button.style.position = 'fixed';
  button.style.zIndex = '9999';
  button.style.padding = '10px 15px';
  button.style.backgroundColor = '#4CAF50';
  button.style.color = 'white';
  button.style.border = 'none';
  button.style.borderRadius = '5px';
  button.style.cursor = 'move';
  button.style.fontSize = '14px';
  button.style.boxShadow = '0 2px 6px rgba(0,0,0,0.2)';

  // Load saved position if it exists
  chrome.storage.local.get('buttonPosition', (data) => {
    if (data.buttonPosition) {
      const { top, left } = data.buttonPosition;
      button.style.top = `${top}px`;
      button.style.left = `${left}px`;
      button.style.right = 'unset';
    } else {
      // Default position
      button.style.top = '100px';
      button.style.right = '20px';
    }

    document.body.appendChild(button);
    makeButtonDraggable(button);

    button.addEventListener('click', (e) => {
      if (!button.dragging) {
        createTradeCaptureModal();
      }
    });
  });
}

function makeButtonDraggable(button) {
  let isDragging = false;
  let startX, startY;
  let currentX, currentY;
  const dragThreshold = 5; // Pixels - how much movement counts as "real drag"

  button.addEventListener('mousedown', (e) => {
    startX = e.clientX;
    startY = e.clientY;
    isDragging = true;
    button.dragging = false; // Start as not dragging yet
    e.preventDefault();
  });

  document.addEventListener('mousemove', (e) => {
    if (!isDragging) return;

    currentX = e.clientX;
    currentY = e.clientY;
    
    const deltaX = Math.abs(currentX - startX);
    const deltaY = Math.abs(currentY - startY);

    // If mouse moved enough, then treat as dragging
    if (deltaX > dragThreshold || deltaY > dragThreshold) {
      button.dragging = true;
      button.style.left = `${e.clientX - button.offsetWidth / 2}px`;
      button.style.top = `${e.clientY - button.offsetHeight / 2}px`;
      button.style.right = 'unset'; // unlock right side
      
      // Save to chrome.storage
      chrome.storage.local.set({ buttonPosition: { top, left } });
    }
  });

  document.addEventListener('mouseup', (e) => {
    if (!isDragging) return;
    isDragging = false;
    setTimeout(() => {
      button.dragging = false;
    }, 100); // Small delay to avoid triggering click right after drag
  });
}

function injectModalStyles() {
  if (document.getElementById('trade-capture-style')) return;

  const style = document.createElement('style');
  style.id = 'trade-capture-style';
  style.innerHTML = `
    @keyframes fadeIn {
      from { opacity: 0; }
      to { opacity: 1; }
    }
    @keyframes slideDown {
      from { transform: translateY(-20px); opacity: 0; }
      to { transform: translateY(0); opacity: 1; }
    }
    @keyframes scaleIn {
      from { transform: scale(0.95); opacity: 0; }
      to { transform: scale(1); opacity: 1; }
    }
    .trade-modal-overlay {
      position: fixed;
      top: 0;
      left: 0;
      width: 100%;
      height: 100%;
      background-color: rgba(0, 0, 0, 0.5);
      display: flex;
      justify-content: center;
      align-items: center;
      z-index: 9999;
      animation: fadeIn 0.2s ease-out;
      backdrop-filter: blur(4px);
    }
    .trade-modal {
      background: white;
      padding: 24px;
      border-radius: 12px;
      width: 360px;
      font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, Oxygen, Ubuntu, Cantarell, sans-serif;
      box-shadow: 0 8px 32px rgba(0, 0, 0, 0.1);
      animation: scaleIn 0.3s ease-out;
      position: relative;
    }
    .trade-modal-header {
      margin-bottom: 20px;
      display: flex;
      justify-content: space-between;
      align-items: center;
    }
    .trade-modal-title {
      font-size: 20px;
      font-weight: 600;
      color: #1a1a1a;
      margin: 0;
    }
    .trade-modal-close {
      background: none;
      border: none;
      font-size: 20px;
      color: #666;
      cursor: pointer;
      padding: 4px;
      border-radius: 4px;
      transition: all 0.2s;
    }
    .trade-modal-close:hover {
      background: #f5f5f5;
      color: #333;
    }
    .trade-form-group {
      margin-bottom: 16px;
      display: flex;
      flex-direction: column;
    }
    .trade-form-label {
      display: block;
      margin-bottom: 6px;
      font-size: 14px;
      font-weight: 500;
      color: #333;
      line-height: 1.4;
    }
    .trade-form-input,
    .trade-form-select,
    .trade-form-textarea {
      width: 100%;
      padding: 10px 12px;
      border: 1px solid #ddd;
      border-radius: 6px;
      font-size: 14px;
      transition: all 0.2s;
      box-sizing: border-box;
      line-height: 1.4;
      font-family: inherit;
    }
    .trade-form-input:focus,
    .trade-form-select:focus,
    .trade-form-textarea:focus {
      border-color: #4CAF50;
      outline: none;
      box-shadow: 0 0 0 2px rgba(76, 175, 80, 0.1);
    }
    .trade-form-textarea {
      min-height: 80px;
      resize: vertical;
      line-height: 1.5;
    }
    .trade-form-select {
      background-color: white;
      cursor: pointer;
      appearance: none;
      background-image: url("data:image/svg+xml;charset=UTF-8,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 24 24' fill='none' stroke='currentColor' stroke-width='2' stroke-linecap='round' stroke-linejoin='round'%3e%3cpolyline points='6 9 12 15 18 9'%3e%3c/polyline%3e%3c/svg%3e");
      background-repeat: no-repeat;
      background-position: right 12px center;
      background-size: 16px;
      padding-right: 36px;
    }
    .trade-modal-footer {
      margin-top: 24px;
      display: flex;
      gap: 12px;
    }
    .trade-button {
      flex: 1;
      padding: 12px;
      border: none;
      border-radius: 6px;
      font-size: 14px;
      font-weight: 500;
      cursor: pointer;
      transition: all 0.2s;
    }
    .trade-button-primary {
      background: #4CAF50;
      color: white;
    }
    .trade-button-primary:hover {
      background: #43A047;
    }
    .trade-button-secondary {
      background: #f5f5f5;
      color: #333;
    }
    .trade-button-secondary:hover {
      background: #e0e0e0;
    }
    .trade-button:disabled {
      opacity: 0.7;
      cursor: not-allowed;
    }
  `;
  document.head.appendChild(style);
}

function createTradeCaptureModal() {
  if (document.getElementById('trade-capture-overlay')) return;

  const overlay = document.createElement('div');
  overlay.id = 'trade-capture-overlay';
  overlay.className = 'trade-modal-overlay';

  const modal = document.createElement('div');
  modal.id = 'trade-capture-modal';
  modal.className = 'trade-modal';

  modal.innerHTML = `
    <div class="trade-modal-header">
      <h3 class="trade-modal-title">Capture Trade</h3>
      <button class="trade-modal-close" id="close-modal">×</button>
    </div>
    <div class="trade-form-group">
      <label class="trade-form-label" for="trade-direction">Direction</label>
      <select id="trade-direction" class="trade-form-select">
        <option value="Long">Long</option>
        <option value="Short">Short</option>
      </select>
    </div>
    <div class="trade-form-group">
      <label class="trade-form-label" for="trade-entry">Entry Price</label>
      <input type="number" id="trade-entry" class="trade-form-input" step="any" placeholder="Enter entry price"/>
    </div>
    <div class="trade-form-group">
      <label class="trade-form-label" for="trade-sl">Stop Loss</label>
      <input type="number" id="trade-sl" class="trade-form-input" step="any" placeholder="Enter stop loss"/>
    </div>
    <div class="trade-form-group">
      <label class="trade-form-label" for="trade-tp">Take Profit</label>
      <input type="number" id="trade-tp" class="trade-form-input" step="any" placeholder="Enter take profit"/>
    </div>
    <div class="trade-form-group">
      <label class="trade-form-label" for="trade-notes">Notes</label>
      <textarea id="trade-notes" class="trade-form-textarea" placeholder="Add any notes about this trade..."></textarea>
    </div>
    <div class="trade-modal-footer">
      <button id="save-trade" class="trade-button trade-button-primary">Save Trade</button>
      <button id="close-modal-secondary" class="trade-button trade-button-secondary">Cancel</button>
    </div>
  `;

  overlay.appendChild(modal);
  document.body.appendChild(overlay);

  // Add event listeners
  const closeModal = () => {
    overlay.style.animation = 'fadeIn 0.2s ease-out reverse';
    modal.style.animation = 'scaleIn 0.3s ease-out reverse';
    setTimeout(() => overlay.remove(), 200);
  };

  document.getElementById('close-modal').addEventListener('click', closeModal);
  document.getElementById('close-modal-secondary').addEventListener('click', closeModal);
  document.getElementById('save-trade').addEventListener('click', () => {
    const saveButton = document.getElementById('save-trade');
    saveButton.disabled = true;
    saveButton.textContent = 'Saving...';
    saveTradeFromModal().finally(() => {
      closeModal();
    });
  });

  // Close on overlay click
  overlay.addEventListener('click', (e) => {
    if (e.target === overlay) {
      closeModal();
    }
  });

  // Close on Escape key
  document.addEventListener('keydown', (e) => {
    if (e.key === 'Escape' && document.getElementById('trade-capture-overlay')) {
      closeModal();
    }
  });

  // Focus first input
  setTimeout(() => {
    document.getElementById('trade-entry').focus();
  }, 100);
}

async function sendTradeToWebDashboard(trade) {
  try {
    // Step 1: Retrieve stored API key
    const { apiKey } = await new Promise((resolve) => {
      chrome.storage.local.get(['apiKey'], resolve);
    });

    if (!apiKey) {
      console.error('❌ API key not found. Please set your API key in the extension settings.');
      return;
    }

    // Step 2: Send POST request to backend
    const response = await fetch('https://tvjournal-two.vercel.app/api/trades', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'Authorization': `Bearer ${apiKey}`
      },
      body: JSON.stringify(trade)
    });

    const result = await response.json();

    // Step 3: Handle backend response
    if (response.ok && result.success) {
      console.log('✅ Trade saved to dashboard.');
    } else {
      console.error('❌ Backend error:', result.error || 'Unknown error');
    }
  } catch (err) {
    console.error('❌ Failed to send trade:', err);
  }
}

async function captureTradingViewChart() {
  return new Promise((resolve, reject) => {
    try {
      const chartContainer = document.querySelector('.chart-container') ||
                           document.querySelector('[data-name="chart-container"]') ||
                           document.querySelector('.tv-chart-view');
      
      if (!chartContainer) {
        reject(new Error('Chart container not found'));
        return;
      }

      // Configure html2canvas options
      const options = {
        useCORS: true,
        allowTaint: true,
        foreignObjectRendering: true,
        scale: window.devicePixelRatio || 1,
        logging: false,
        backgroundColor: null,
        onclone: (clonedDoc) => {
          // Try to find the canvas in the cloned document
          const clonedCanvas = clonedDoc.querySelector('canvas');
          if (clonedCanvas) {
            // Set crossOrigin attribute
            clonedCanvas.crossOrigin = 'anonymous';
          }
        }
      };

      html2canvas(chartContainer, options)
        .then(canvas => {
          try {
            // Try to get the image data
            const imageBase64 = canvas.toDataURL('image/png', 1.0);
            if (!imageBase64 || imageBase64 === 'data:,') {
              throw new Error('Failed to generate image data');
            }
            resolve(imageBase64);
          } catch (err) {
            console.error('Canvas to image conversion failed:', err);
            // Fallback: Try to capture the container as a screenshot
            const containerRect = chartContainer.getBoundingClientRect();
            const screenshot = {
              width: containerRect.width,
              height: containerRect.height,
              top: containerRect.top,
              left: containerRect.left
            };
            
            // Send message to background script to capture the area
            chrome.runtime.sendMessage({
              type: 'CAPTURE_VISIBLE_TAB',
              area: screenshot
            }, (response) => {
              if (chrome.runtime.lastError) {
                reject(new Error('Failed to capture screenshot: ' + chrome.runtime.lastError.message));
              } else if (response && response.imageData) {
                resolve(response.imageData);
              } else {
                reject(new Error('Failed to capture screenshot'));
              }
            });
          }
        })
        .catch(err => {
          console.error('html2canvas failed:', err);
          reject(err);
        });
    } catch (err) {
      console.error('Chart capture failed:', err);
      reject(err);
    }
  });
}

async function saveTradeFromModal() {
  try {
    let imageBase64 = null;
    try {
      imageBase64 = await captureTradingViewChart();
    } catch (captureError) {
      console.warn('Chart capture failed:', captureError);
      // Continue without the screenshot
    }

    const direction = document.getElementById('trade-direction').value;
    const entry_price = document.getElementById('trade-entry').value;
    const stop_loss = document.getElementById('trade-sl').value;
    const take_profit = document.getElementById('trade-tp').value;
    const notes = document.getElementById('trade-notes').value;
    let symbol = 'Unknown';
    const pageTitle = document.title;
    
    if (pageTitle.includes(' ')) {
      symbol = pageTitle.split(' ')[0].trim();
    }

    const tradeData = {
      symbol,
      direction,
      entry_price,
      stop_loss,
      take_profit,
      notes,
      timestamp: new Date().toISOString(),
      ...(imageBase64 && { imageBase64 }) // Only include imageBase64 if it exists
    };

    await sendTradeToWebDashboard(tradeData);
  } catch (err) {
    console.error('Failed to save trade:', err);
    const saveButton = document.getElementById('save-trade');
    saveButton.disabled = false;
    saveButton.textContent = 'Save Trade';
    alert('Failed to save trade. Please try again.');
  }
}

// Call it when page loads
injectModalStyles();

// Inject button after page load
window.addEventListener('load', () => {
  setTimeout(createCaptureButton, 3000); // Wait 3s to ensure TradingView UI loads
});

